#include "testlib.h"
#include <vector>

using namespace std;

vector<string> split(string &s, char d) {
    vector<string> ans;
    string cur;
    for (char c : s) {
        if (c == d) {
            if (!cur.empty()) {
                ans.push_back(cur);
                cur = "";
            }
        } else {
            cur.push_back(c);
        }
    }
    if (!cur.empty()) {
        ans.push_back(cur);
        cur = "";
    }
    return ans;
}

inline bool isn(char c) {
    return '0' <= c && c <= '9';
}

inline bool isop(char c, int t) {
    if (t == 1) {
        return c == '+' || c == '=';
    } else if (t == 2) {
        return c == '+' || c == '=' || c == '*';
    }

    quitf(_fail, "something's wrong with type here");
}

long long count_mult(string &s, long long mod) {
    auto v = split(s, '*');
    long long ans = 1;
    for (string &t : v) {
        long long cur = 0;
        for (char x : t) {
            if (!isn(x)) {
                quitf(_wa, "some weird characters in output");
            }
            cur *= 10;
            cur += x - '0';
            cur %= mod;
        }
        ans *= cur;
        ans %= mod;
    }
    return ans;
}

long long cnt(string &s, long long mod) {
    auto v = split(s, '+');
    long long ans1 = 0;
    for (string &t : v) {
        ans1 += count_mult(t, mod);
        ans1 %= mod;
    }
    return ans1;
}

void check(string s, string res, int k, int m, int t) {
    string nums = "";
    bool lastop = false;
    for (int i = 0; i < res.length(); i++) {
        if (isn(res[i])) {
            nums.push_back(res[i]);
            lastop = false;
        } else if (isop(res[i], t)) {
            if (i == 0) {
                quitf(_wa, "operation before first digit");
            }
            if (lastop) {
                quitf(_wa, "two operations in a row");
            }
            lastop = true;
        } else {
            quitf(_wa, "illegal character %c", s[i]);
        }
    }
    
    if (lastop) {
        quitf(_wa, "extra operation");
    }

    if (s != nums) {
        quitf(_wa, "wrong sequence of digits");
    }

    auto v = split(res, '=');
    if (v.size() != k + 1) {
        quitf(_wa, "%d equality signs instead of %d", (int) v.size() - 1, k);
    }

    vector<long long> vals(k + 1);
    for (int i = 0; i < k + 1; i++) {
        vals[i] = cnt(v[i], m);
        if (vals[i] != vals[0]) {
            quitf(_wa, "values are different");
        }
    }
}

string fl = "Fail";

int main(int argc, char *argv[]) {
    registerTestlibCmd(argc, argv);

    int mt = inf.readInt();
    for (int i = 0; i < mt; i++) {
        int n = inf.readInt();
        int k = inf.readInt();
        int m = inf.readInt();
        int t = inf.readInt();

        string s = inf.readToken();

        string ok = ans.readToken();
        string cur = ouf.readToken();

        if (cur == fl && ok != fl) {
            quitf(_wa, "contestant hasn't found the answer");
        }

        if (cur == fl && ok == fl) {
            continue; // no solution
        }

        check(s, cur, k, m, t);

        if (ok == fl) {
            quitf(_fail, "jury hasn't found the answer");
        }
    }

    quitf(_ok, "%d tests", mt);
}
