import random
import math
random.seed('personal02/receipt')

class TestGen:
    def __init__(self):
        self.test_number = 0

    def generate_samples(self):
        print('Generating samples...')
        self.print_test(3, [1, 3])
        self.print_test(5, [1, 2, 3, 4, 5, 5])

    def generate_min_tests(self):
        self.print_test(1, [1])
        self.print_test(1, [0, 1])
        self.print_test(1, [1, 1])
        self.print_test(1, [0, 0, 0, 0, 1, 1, 1, 1, 1, 1])
        self.print_test(2, [0, 0, 0, 0, 1, 1, 1, 1, 2, 2])
        self.print_test(2, [0, 0, 0, 1, 2, 2, 2, 2, 2, 2, 2])
        self.print_test(5, [1, 2, 3, 4, 5])

    def generate_random_sorted_array(self, n, m, equal=True):
        def update(a):
            b = [0] * m
            for i in range(len(a)):
                b[a[i] - 1] += 1
            c = [0] * m
            c[0] = b[0]
            for i in range(1, m):
                c[i] = c[i - 1] + b[i]
            return c
    
        a = []
        prev = 0
        for i in range(n):
            if equal:
                x = random.randint(max(1, prev), m)
            else:
                x = random.randint(prev + 1, m - (n - i - 1))
            prev = x
            a.append(x)
        self.print_test(n, update(a))

    def generate_first_group_of_tests(self):
        print('Generating subtask1...')
        self.generate_min_tests()
        for n in [5, 10, 50, 100, 1000, 10**4]:
            for _ in range(3):
                m = random.randint(n, min(10**5, 3 * n))
                self.generate_random_sorted_array(n, m, False)
        self.print_test(10**5, [i for i in range(1, 10**5 + 1)])
        self.print_test(10**5 - 1, [i for i in range(1, 10**5 - 1)] + [10**5 - 1, 10**5 - 1])

    def generate_second_group_of_tests(self):
        print('Generating subtask2...')
        for n in [5, 10, 50, 100, 1000, 10**5]:
            for _ in range(3):
                m = random.randint(1, min(10**5, 3 * n))
                self.generate_random_sorted_array(n, m)
        self.print_test(10**5 - 1, [i for i in range(1, 10**5)])
        self.print_test(10**5 - 2, [i for i in range(1, 10**5 - 3)] + [10**5 - 2, 10**5 - 2])
        self.print_test(10**5, [10**5])
        x = random.randint(1, 10**5 - 1)
        self.print_test(1, [0] * x + [1] * (10**5 - x))
        self.print_test(1, [0] * (10**5 - 1) + [1])
        self.print_test(1, [1] * (10**5))
        self.print_test(1000, [1000])

    def print_test(self, n, b):
        self.test_number += 1
        print('Generating test %d' % self.test_number)
        
        test_name = '{0:0=2d}'.format(self.test_number)
        test_file = open(test_name, 'w')
        print(n, len(b), file=test_file)
        print(' '.join(map(str, b)), file=test_file)
        test_file.close()

    def generate_all_tests(self):
        #Samples
        self.generate_samples() 
        
        #First group of tests
        self.generate_first_group_of_tests()
        
        #Second group of tests
        self.generate_second_group_of_tests()

import time
start = time.time()
writer = TestGen()
writer.generate_all_tests()
finish = time.time()
print('All tests were generated. Elapsed time: ', finish - start)
